#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
@author: wuyue
@contact: wuyue92tree@163.com
@software: PyCharm
@file: middlewares.py
@create at: 2018-06-26 18:21

这一行开始写关于本文件的说明与解释

通过redis hash表记录站点cookie

key为 cookie_pool:SITE (SITE需要在settings中指定)
field为 账号cookie的唯一标识，可以是username，id等，具体自行约定
value为 cookie值，必须为json格式

"""

import json
import random
from scrapy.downloadermiddlewares.retry import RetryMiddleware
from scrapy_redis.connection import get_redis_from_settings
from crwy.utils.data.RedisHash import RedisHash
from crwy.exceptions import CrwyScrapyPlugsException


class CookieMiddleware(RetryMiddleware):
    """
    cookie_pool
        eg: '{"a": 1, "b": "aaa"}'
    """

    def __init__(self, settings):
        super(CookieMiddleware, self).__init__(settings)
        self.site = settings.get('SITE', None)
        if not self.site:
            raise CrwyScrapyPlugsException('SITE_NOT_SET')

        self.server = get_redis_from_settings(settings)

        self.h = RedisHash(
            'cookie_pool:{}'.format(self.site),
            server=self.server
        )

    def process_request(self, request, spider):
        """
        cookie_user不为空时，获取cookie_user对应的cookie
        keep_cookie_user为True时，将设置cookie_user，并传递到子请求
        :param request:
        :param spider:
        :return:
        """
        if request.meta.get('cookie_user', ''):
            user = request.meta.get('cookie_user')
        else:
            users = self.h.hkeys()
            if len(users) > 0:
                user = random.choice(users)
                if request.meta.get('keep_cookie_user', False) is True:
                    request.meta['cookie_user'] = user
            else:
                raise CrwyScrapyPlugsException(
                    'no user in cookie_pool:{}'.format(self.site))
        cookie = self.h.hget(user)
        if cookie:
            # 字典存入redis，取出时未string，通过eval进行还原
            request.cookies = eval(cookie)
            spider.logger.debug('get_cookie_success: {}'.format(
                user.decode('utf-8')))
        else:
            spider.logger.warning('get_cookie_failed: {}'.format(
                user.decode('utf-8')))



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# Author: wuyue
# Email: wuyue92tree@163.com


import logging
import logging.config
import logging.handlers
from crwy.exceptions import CrwyException

try:
    import ConfigParser
except ImportError:
    from configparser import ConfigParser

DEFAULT_LOGGER_CONF = './conf/logger.conf'

try:
    logging.config.fileConfig(DEFAULT_LOGGER_CONF)
except:
    pass


def _install_handlers_custom(cp, formatters, log_path):
    """Install and return handlers"""
    hlist = cp.get("handlers", "keys")
    if not len(hlist):
        return {}
    hlist = hlist.split(",")
    hlist = logging.config._strip_spaces(hlist)
    handlers = {}
    fixups = []  # for inter-handler references
    for hand in hlist:
        sectname = "handler_%s" % hand
        klass = cp.get(sectname, "class")
        opts = cp.options(sectname)
        if "formatter" in opts:
            fmt = cp.get(sectname, "formatter")
        else:
            fmt = ""
        try:
            klass = eval(klass, vars(logging))
        except (AttributeError, NameError):
            klass = logging.config._resolve(klass)
        args = cp.get(sectname, "args")
        args = eval(args, vars(logging))

        # 修改args中的path参数
        if isinstance(args[0], str):
            args = tuple([log_path] + list(args)[1:])

        h = klass(*args)

        if "level" in opts:
            level = cp.get(sectname, "level")
            h.setLevel(logging._levelNames[level])
        if len(fmt):
            h.setFormatter(formatters[fmt])
        if issubclass(klass, logging.handlers.MemoryHandler):
            if "target" in opts:
                target = cp.get(sectname, "target")
            else:
                target = ""
            if len(target):
                fixups.append((h, target))
        handlers[hand] = h

    for h, t in fixups:
        h.setTarget(handlers[t])
    return handlers


def fileConfigWithLogPath(fname=DEFAULT_LOGGER_CONF,
                          log_path=None,
                          defaults=None,
                          disable_existing_loggers=True):
    """
    通过拦截重写handler的方式传入log_path，实现日志位置修改
    """
    if not log_path:
        raise CrwyException('Please setup <log_path> first!')

    cp = ConfigParser.ConfigParser(defaults)
    if hasattr(fname, 'readline'):
        cp.readfp(fname)
    else:
        cp.read(fname)
    try:
        formatters = logging.config._create_formatters(cp)
    except ConfigParser.NoSectionError:
        raise CrwyException('Please make sure fname: "%s" is exist.' % fname)

    logging._acquireLock()
    try:
        logging._handlers.clear()
        del logging._handlerList[:]
        # Handlers add themselves to logging._handlers
        handlers = _install_handlers_custom(cp, formatters, log_path)
        logging.config._install_loggers(cp, handlers, disable_existing_loggers)
    finally:
        logging._releaseLock()


class Logger(object):
    @staticmethod
    def file_logger():
        return logging.getLogger('fileLogger')

    @staticmethod
    def rt_logger():
        return logging.getLogger('rtLogger')

    @staticmethod
    def timed_rt_logger():
        return logging.getLogger('timedRtLogger')

    @staticmethod
    def extra_logger(name=None):
        return logging.getLogger(name)




#!/usr/local/bin/python
# -*- coding: utf-8 -*-
# author: wuyue92tree@163.com

from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from sqlalchemy.ext.declarative import declarative_base
from crwy.decorates import cls2singleton

Base = declarative_base()


@cls2singleton
class SqlalchemyHandle(object):
    """
    以ORM的方式连接数据库
    """

    def __init__(self, db_url, **kwargs):
        self.engine = create_engine(db_url, **kwargs)
        DBSession = sessionmaker(bind=self.engine)
        self.session = DBSession()

    def init_table(self):
        return Base.metadata.create_all(self.engine)

    def delete_table(self):
        return Base.metadata.drop_all(self.engine)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
@author: wuyue
@contact: wuyue92tree@163.com
@software: PyCharm
@file: dingding_robot.py
@create at: 2017-10-24 10:57

这一行开始写关于本文件的说明与解释
"""

import json

from crwy.spider import BaseSpider
from crwy.exceptions import CrwyExtendException


class DingDingRobot(BaseSpider):
    def __init__(self, access_token=None,
                 api_url="https://oapi.dingtalk.com/robot/send?access_token="):
        super(DingDingRobot, self).__init__()
        if not api_url:
            raise CrwyExtendException('access_token unset.')
        self.api_url = api_url
        self.header = {'Content-Type': 'application/json'}
        self.access_token = access_token
        self.html_downloader.session.headers = self.header

    def send_text(self, content, at_mobiles=list(), is_at_all=False):
        try:
            data = {
                "text": {
                    "content": content
                },
                "msgtype": "text",
                "at": {
                    "isAtAll": is_at_all,
                    "atMobiles": at_mobiles
                }
            }

            res = self.html_downloader.download(
                self.api_url + self.access_token,
                method='POST',
                data=json.dumps(data))
            return res
        except Exception as e:
            raise CrwyExtendException(e)

    def send_markdown(self, title, content, at_mobiles=list(),
                      is_at_all=False):
        try:
            data = {
                "msgtype": "markdown",
                "markdown": {
                    "title": title,
                    "text": content
                },
                "at": {
                    "atMobiles": at_mobiles,
                    "isAtAll": is_at_all
                }
            }

            res = self.html_downloader.download(
                self.api_url + self.access_token,
                method='POST',
                data=json.dumps(data))
            return res
        except Exception as e:
            raise CrwyExtendException(e)

    def send_action_card(self, title, content, hide_avatar="0",
                         btn_oriengtation="0", single_title="阅读全文",
                         single_url="#"):
        try:
            data = {
                "actionCard": {
                    "title": title,
                    "text": content,
                    "hideAvatar": hide_avatar,
                    "btnOrientation": btn_oriengtation,
                    "singleTitle": single_title,
                    "singleURL": single_url
                },
                "msgtype": "actionCard"
            }
            res = self.html_downloader.download(
                self.api_url + self.access_token,
                method='POST',
                data=json.dumps(data))
            return res
        except Exception as e:
            raise CrwyExtendException(e)

    def send_feed_card(self, links):
        """

        :param links: array[{'title':'', 'messageURL':'', 'picURL':''}]
        :return:
        """
        try:
            data = {
                "feedCard": {
                    "links": links
                },
                "msgtype": "feedCard"
            }
            res = self.html_downloader.download(
                self.api_url + self.access_token,
                method='POST',
                data=json.dumps(data))
            return res
        except Exception as e:
            raise CrwyExtendException(e)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# author: wuyue92tree@163.com

import requests


class HtmlDownloader(object):
    """ 下载器 """

    def __init__(self):
        self.session = requests.session()

    def download(self, url, method='GET', single=False, timeout=60,
                 **kwargs):
        """
        请求页面
        :param url: 目标地址
        :param method: 请求方式
        :param single: 为True时，不使用session
        :param timeout: 初始化超时时间
        :param kwargs: 更多requests参数
        :return: 返回requests session对象
        """

        if single is False:
            if method.upper() == 'GET':
                return self.session.get(url, timeout=timeout, **kwargs)
            return self.session.post(url, timeout=timeout, **kwargs)
        else:
            if method.upper() == 'GET':
                return requests.get(url, timeout=timeout, **kwargs)
            return requests.post(url, timeout=timeout, **kwargs)

    def download_file(self, url, method='GET', single=False, timeout=180,
                      save_path='./data/', file_name=None, **kwargs):
        """
        请求文件
        :param url: 目标地址
        :param method: 请求方式
        :param single: 为True时，不使用session
        :param timeout: 初始化超时时间
        :param save_path: 保存路径
        :param file_name: 文件名称，默认为空
        :param kwargs: 更多requests参数
        :return: 返回保存路径
        """
        if not file_name:
            file_name = url.split('/')[-1]
        tmp = self.download(url, method=method, single=single,
                            timeout=timeout,
                            stream=True, **kwargs)
        with open(save_path + file_name, 'wb') as f:
            for chunk in tmp.iter_content(chunk_size=1024):
                if chunk:  # filter out keep-alive new chunks
                    f.write(chunk)
                    f.flush()
            f.close()
        return save_path + file_name



#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
@author: wuyue
@contact: wuyue92tree@163.com
@software: PyCharm
@file: exceptions.py
@create at: 2017-12-13 14:14

这一行开始写关于本文件的说明与解释
"""


class CrwyException(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


class CrwyImportException(CrwyException):
    pass


class CrwyKafkaException(CrwyException):
    pass


class CrwyMnsException(CrwyException):
    pass


class CrwyDbException(CrwyException):
    pass


class CrwyExtendException(CrwyException):
    pass


class CrwyCookieValidException(CrwyException):
    pass


class CrwyScrapyPlugsException(CrwyException):
    pass



#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
@author: wuyue
@contact: wuyue92tree@163.com
@software: PyCharm
@file: mysql.py
@create at: 2017-12-15 14:20

这一行开始写关于本文件的说明与解释
"""

from crwy.exceptions import CrwyImportException, CrwyDbException
from crwy.decorates import cls2singleton

try:
    import pymysql
except ImportError:
    raise CrwyImportException(
        "You should install pymysql first! try: pip install "
        "pymysql")
try:
    from DBUtils.PersistentDB import PersistentDB
except ImportError:
    raise CrwyImportException(
        "You should install DBUtils first! try: pip install "
        "dbutils")


@cls2singleton
class MysqlHandle(object):
    def __init__(self, **kwargs):
        self._mysql_pool = PersistentDB(pymysql, **kwargs)

    def query_by_sql(self, sql):
        conn = self._mysql_pool.connection()
        cur = conn.cursor()
        try:
            cur.execute(sql)
            result = cur.fetchall()
            return result
        except Exception as e:
            raise CrwyDbException(e)
        finally:
            cur.close()
            conn.close()

    def save(self, sql, data, many=False, get_last_insert_id=False):
        conn = self._mysql_pool.connection()
        cur = conn.cursor()
        try:
            if many is False:
                cur.execute(sql, data)
            else:
                cur.executemany(sql, data)
            conn.commit()

            if get_last_insert_id is False:
                return

            cur.execute("select last_insert_id() as id")
            res = cur.fetchone()
            if isinstance(res, tuple):
                return res[0]
            elif isinstance(res, dict):
                return res.get('id')
            else:
                return res

        except Exception as e:
            raise CrwyDbException(e)
        finally:
            cur.close()
            conn.close()



#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
@author: wuyue
@contact: wuyue92tree@163.com
@software: PyCharm
@file: pipelines.py
@create at: 2018-06-15 15:26

这一行开始写关于本文件的说明与解释
"""

import logging
from pymysql.cursors import DictCursor
from sqlalchemy.ext.compiler import compiles
from sqlalchemy.sql.expression import Insert
from crwy.utils.sql.mysql import MysqlHandle
from crwy.utils.sql.sqlalchemy_m import SqlalchemyHandle
from crwy.exceptions import CrwyScrapyPlugsException


class MysqlSavePipeline(object):
    def __init__(self, db_name=None, db_host=None, db_port=None,
                 db_username=None, db_password=None, db_charset=None,
                 db_cursorclass=None):
        self.db_name = db_name
        self.db_host = db_host
        self.db_port = db_port
        self.db_username = db_username
        self.db_password = db_password
        self.db_charset = db_charset
        self.db_cursorclass = db_cursorclass
        self.logger = logging.getLogger(__name__)
        self.mysql_handle = None

    @classmethod
    def from_crawler(cls, crawler):
        """
        loading mysql settings
        :param crawler: 
        :return: 
        """
        settings = crawler.settings
        db_name = settings.get('MYSQL_DB_NAME', '')
        db_host = settings.get('MYSQL_DB_HOST', '127.0.0.1')
        db_port = settings.getint('MYSQL_DB_PORT', 3306)
        db_username = settings.get('MYSQL_DB_USERNAME', 'root')
        db_password = settings.get('MYSQL_DB_PASSWORD', '123456')
        db_charset = settings.get('MYSQL_DB_CHARSET', 'utf8')
        db_cursorclass = settings.get('MYSQL_DB_CURSORCLASS', DictCursor)
        return cls(db_name=db_name, db_host=db_host, db_port=db_port,
                   db_username=db_username,
                   db_password=db_password,
                   db_charset=db_charset,
                   db_cursorclass=db_cursorclass)

    def process_item(self, item, spider):
        self.insert_db(item)
        return item

    def open_spider(self, spider):
        self.mysql_handle = MysqlHandle(
            host=self.db_host,
            port=self.db_port,
            user=self.db_username,
            password=self.db_password,
            db=self.db_name,
            charset=self.db_charset,
            cursorclass=self.db_cursorclass
        )

    def insert_db(self, item):
        """        
        -----------------------------------
        Do something here with mysql_handle
        -----------------------------------
        
        eg:
        sql = None
        data = None
        last_insert_id = self.mysql_handle.save(
            sql, data, get_last_insert_id=True)
        self.logger.info('item saved succcess to mysql: %s' % last_insert_id)
        """
        pass


@compiles(Insert)
def append_string(insert, compiler, **kw):
    s = compiler.visit_insert(insert, **kw)
    if 'append_string' in insert.kwargs:
        return s + " " + insert.kwargs['append_string']
    return s


class SqlalchemySavePipeline(object):
    def __init__(self, db_url, echo=True):
        self.db_url = db_url
        self.echo = echo
        self.sqlalchemy_handle = None
        self.logger = logging.getLogger(__name__)

    @classmethod
    def from_crawler(cls, crawler):
        """
        loading sqlalchemy settings
        :param crawler:
        :return:
        """
        settings = crawler.settings
        db_url = settings.get('SQLALCHEMY_URI')
        echo = settings.getbool('SQLALCHEMY_ECHO')
        if not db_url:
            raise CrwyScrapyPlugsException('SQLALCHEMY_URI must be setup.')
        return cls(db_url, echo)

    def process_item(self, item, spider):
        self.insert_db(item)
        return item

    def open_spider(self, spider):
        self.sqlalchemy_handle = SqlalchemyHandle(
            db_url=self.db_url, echo=self.echo)
        self.sqlalchemy_handle.init_table()

    def insert_db(self, item):
        """
        -----------------------------------
        Do something here with sqlalchemy_handle
        -----------------------------------

        eg:
        self.sqlalchemy_handle.session.execute(
            Test.__table__.insert(), item
        )
        self.sqlalchemy_handle.session.commit()
        self.logger.info('sqlachemy inserted success.')
        """
        pass

    def close_spider(self, spider):
        self.sqlalchemy_handle.session.close()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# author: wuyue92tree@163.com

from crwy.utils.no_sql.redis_m import get_redis_client


class RedisHash(object):
    """Simple Hash with Redis Backend"""

    def __init__(self, name, server=None, **redis_kwargs):
        """
        The default connection parameters are:
        host='localhost', port=6379, db=0
        """
        if server:
            self.__db = server
        else:
            self.__db = get_redis_client(**redis_kwargs)
        self.key = name

    def hget(self, item):
        """Get item value."""
        return self.__db.hget(self.key, item)

    def hset(self, item, value):
        """Set item value."""
        return self.__db.hset(self.key, item, value)

    def hexists(self, item):
        """Is item exist."""
        return self.__db.hexists(self.key, item)

    def hlen(self):
        """Return total count."""
        return self.__db.hlen(self.key)

    def hkeys(self):
        return self.__db.hkeys(self.key)

    def clean(self):
        """Empty key"""
        return self.__db.delete(self.key)

    def db(self):
        return self.__db



#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
@author: wuyue
@contact: wuyue92tree@163.com
@software: PyCharm
@file: decorates.py
@create at: 2017-12-07 09:47

这一行开始写关于本文件的说明与解释
"""

import functools
from crwy.exceptions import CrwyCookieValidException


def cls2singleton(cls, *args, **kwargs):
    """
    将类转换为单例模式
    :param cls: 
    :param args: 
    :param kwargs: 
    :return: 
    """
    instances = {}

    def _singleton(*args, **kwargs):
        if kwargs.pop('cls_singleton', True) is False:
            return cls(*args, **kwargs)
        if cls not in instances:
            instances[cls] = cls(*args, **kwargs)
        return instances[cls]

    return _singleton


def cls_catch_exception(func):
    """
    该装饰器用于捕捉类方法异常
    1. 未出现异常，直接return方法执行结果
    2. 出现异常，则先将异常记入日志，再抛出异常
    :param func:
    :return:
    """

    @functools.wraps(func)
    def wrapper(self, *args, **kwargs):
        try:
            return func(self, *args, **kwargs)
        except Exception as e:
            self.logger.exception(e)
            raise e

    return wrapper


def cls_refresh_cookie(func):
    """
    该装饰器用于捕捉类方法异常 CrwyCookieValidException
    1. 未出现异常，直接return方法执行结果
    2. 出现异常，则先调用self.get_cookie()进行cookie刷新，若cookie刷新成功，
    直接return返回
    :param func:
    :return:
    """

    @functools.wraps(func)
    def wrapper(self, *args, **kwargs):
        try:
            return func(self, *args, **kwargs)
        except CrwyCookieValidException as e:
            if not self.get_cookie():
                self.logger.warning("Func[%s]: cookie更新失败." % func.__name__)
                raise e
            self.logger.info("Func[%s]: cookie更新成功." % func.__name__)
            return func(self, *args, **kwargs)

    return wrapper



# -*- coding: utf-8 -*-

import sys
from bs4 import BeautifulSoup
try:
    import PyV8
except ImportError:
    pass


class HtmlParser(object):
    """ 解析器 """
    @staticmethod
    def parser(response):
        """
        utf-8字符处理
        :param response: 待处理字符串
        :return: 返回bs对象
        """
        if response is None:
            return

        if sys.version_info < (3, ):
            soup = BeautifulSoup(str(response), 'html.parser',
                                 from_encoding='utf-8')
        else:
            soup = BeautifulSoup(str(response), 'html.parser')

        return soup

    @staticmethod
    def gbk_parser(response):
        """
        gbk字符处理
        :param response: 待处理字符串
        :return: 返回bs对象
        """
        if response is None:
            return

        if sys.version_info < (3, ):
            soup = BeautifulSoup(str(response), 'html.parser',
                                 from_encoding='gb18030')
        else:
            soup = BeautifulSoup(str(response), 'html.parser')

        return soup

    @staticmethod
    def jsonp_parser(data):
        """
        非规范json数据处理 {a:1, b:1}
        key非字符串
        :param data: 待处理字符串
        :return: 返回标准json数据
        """
        ctx = PyV8.JSContext()
        ctx.enter()
        ctx.eval("""
            function func() {
              var data = """ + data + """;
              var json_data = JSON.stringify(data);
              return json_data;
            }
        """)
        return ctx.locals.func()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# author: wuyue92tree@163.com

from __future__ import print_function
import os
import sys


class Command(object):
    @staticmethod
    def get_spider_list():
        spider_list = []
        for a, b, c in os.walk('src'):
            for i in c:
                if i == '__init__.py':
                    continue
                if '.pyc' in i:
                    continue
                if '_db' in i:
                    continue
                if '.py' in i:
                    i = i.split('.')[0]
                    spider_list.append(i)
        return spider_list

    def main(self):
        spider_list = self.get_spider_list()

        if len(spider_list) == 0:
            print('No spider found!!!')
            sys.exit(1)

        for spider in spider_list:
            print(spider)
